@@ -2,15 +2,9 @@
 
 //! A cellular automaton simulation library with support for `WebAssembly`.
 
-mod automaton_life_like;
-mod rules;
-
-pub use automaton_life_like::Automaton;
-pub use rules::{
-    bs::RulesetBS,
-    bsc::RulesetBSC,
-    types::{BirthRule, GenerationRule, SurvivalRule},
-};
+pub mod automaton_life_like;
+pub mod rule;
+pub mod ruleset;
 
 #[cfg(target_arch = "wasm32")]
 #[global_allocator]
